"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthIdRefreshToken = exports.schemaAuthIdRefreshTokenClaim = void 0;
const joi_1 = __importDefault(require("joi"));
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const auth_base_token_1 = require("./auth-base-token");
const auth_type_1 = require("./auth-type");
const type_1 = require("../type");
const _uuid = joi_1.default.string().length(21);
exports.schemaAuthIdRefreshTokenClaim = {
    ...auth_type_1.schemaBaseIdToken,
    uuid: _uuid.required(),
    userLoginId: type_1._userLoginId
};
class AuthIdRefreshToken extends auth_base_token_1.AuthBaseToken {
    static self;
    constructor(secret, expiresIn, schema, algorithm) {
        super(secret, expiresIn, schema, type_1.TokenKind.REFRESH_TOKEN, type_1.TokenKind.REFRESH_EXTENDED_TOKEN, algorithm);
    }
    static init(secret, expiresIn, algorithm) {
        this.self = new AuthIdRefreshToken(secret, expiresIn, joi_1.default.object(exports.schemaAuthIdRefreshTokenClaim), algorithm);
        return this;
    }
    static term() {
        this.self = undefined;
    }
    static get instance() {
        return this.self || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'AuthIdRefreshToken initialization is required');
    }
    static get isEnabled() {
        return undefined !== this.self;
    }
    get expiresIn() {
        return this.signOptions.expiresIn;
    }
    static decode(token, kind) {
        return super.baseDecode(token, kind, joi_1.default.object(exports.schemaAuthIdRefreshTokenClaim));
    }
}
exports.AuthIdRefreshToken = AuthIdRefreshToken;
//# sourceMappingURL=auth-id-refresh-token.js.map