import Joi from 'joi';
import { PureAbility } from '@casl/ability';
import { Algorithm, AuthBaseToken } from './auth-base-token';
import { BaseIdToken, Rule } from './auth-type';
import { RoleMetadata } from './auth-type-verify';
import { TokenKind } from '../type';
export type IdToken = BaseIdToken & {
    ability: PureAbility;
    rules?: Rule[];
    rolesMetadata?: RoleMetadata[];
    mfaVerified?: boolean;
};
export type AuthIdTokenClaim = BaseIdToken & {
    uuid: string;
    iat?: number;
    exp?: number;
};
export declare const schemaAuthIdTokenClaim: {
    uuid: Joi.StringSchema<string>;
    iat: Joi.NumberSchema<number>;
    exp: Joi.NumberSchema<number>;
    userId: Joi.NumberSchema<number>;
    userLogin: Joi.StringSchema<string>;
    localeCode: Joi.StringSchema<string>;
    orgId: Joi.NumberSchema<number>;
    appDomain: Joi.StringSchema<string>;
    productId: Joi.NumberSchema<number>;
    extended: Joi.ObjectSchema<any>;
};
export declare class AuthIdToken extends AuthBaseToken<AuthIdTokenClaim> {
    private static self;
    private constructor();
    static init(secret: string, expiresIn: string | number, algorithm?: Algorithm): typeof AuthIdToken;
    static term(): void;
    static get instance(): AuthIdToken;
    static get isEnabled(): boolean;
    get expiresIn(): string | number;
    static decode(token: string, kind?: TokenKind): AuthIdTokenClaim;
}
