"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthIdToken = exports.schemaAuthIdTokenClaim = void 0;
const joi_1 = __importDefault(require("joi"));
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const auth_base_token_1 = require("./auth-base-token");
const auth_type_1 = require("./auth-type");
const type_1 = require("../type");
const _uuid = joi_1.default.string().length(21);
exports.schemaAuthIdTokenClaim = {
    ...auth_type_1.schemaBaseIdToken,
    uuid: _uuid.required(),
    iat: joi_1.default.number(),
    exp: joi_1.default.number()
};
class AuthIdToken extends auth_base_token_1.AuthBaseToken {
    static self;
    constructor(secret, expiresIn, schema, algorithm) {
        super(secret, expiresIn, schema, type_1.TokenKind.TOKEN, type_1.TokenKind.EXTENDED_TOKEN, algorithm);
    }
    static init(secret, expiresIn, algorithm) {
        this.self = new AuthIdToken(secret, expiresIn, joi_1.default.object(exports.schemaAuthIdTokenClaim), algorithm);
        return this;
    }
    static term() {
        this.self = undefined;
    }
    static get instance() {
        return this.self || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'AuthIdToken initialization is required');
    }
    static get isEnabled() {
        return undefined !== this.self;
    }
    get expiresIn() {
        return this.signOptions.expiresIn;
    }
    static decode(token, kind) {
        return super.baseDecode(token, kind, joi_1.default.object(exports.schemaAuthIdTokenClaim));
    }
}
exports.AuthIdToken = AuthIdToken;
//# sourceMappingURL=auth-id-token.js.map