import { AuthToken, BaseIdToken, Rule } from './auth-type';
import { AppDomain } from '../type';
import { RoleModel } from '../model';
export type RoleMetadata = {
    models: Record<string, RoleModel>;
    attributes?: {
        [x: string]: any;
        roleOrgId?: number;
        roleOrgCode?: string;
        roleOrgType?: string;
        roleType?: string;
    };
};
export type VerifyIdToken = BaseIdToken & {
    rules?: Rule[];
    rolesMetadata?: RoleMetadata[];
    mfaVerified?: boolean;
};
export declare const schemaVerifyIdToken: {
    rules: import("joi").ArraySchema<Rule>;
    rolesMetadata: import("joi").ArraySchema<RoleMetadata>;
    mfaVerified: import("joi").BooleanSchema<boolean>;
    userId: import("joi").NumberSchema<number>;
    userLogin: import("joi").StringSchema<string>;
    localeCode: import("joi").StringSchema<string>;
    orgId: import("joi").NumberSchema<number>;
    appDomain: import("joi").StringSchema<string>;
    productId: import("joi").NumberSchema<number>;
    extended: import("joi").ObjectSchema<any>;
};
export type BearerMetadata = {
    orgId: number;
    appDomain: AppDomain;
    productId: number;
    roleType?: string;
    hasAbility?: Rule;
};
export type AuthBearerVerify = (authToken: AuthToken, metadata: BearerMetadata) => Promise<VerifyIdToken>;
export type BasicMetadata = {
    username: string;
    password: string;
    orgId: number;
    appDomain: AppDomain;
    productId: number;
    roleType?: string;
    hasAbility?: Rule;
};
export type AuthBasicVerify = (metadata: BasicMetadata) => Promise<VerifyIdToken>;
