"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getValueByPath = exports.renderValues = exports.renderCondition = exports.generateAbility = exports.schemaRequestMetadata = exports.schemaBaseIdToken = exports._authTokenType = exports.AuthTokenType = void 0;
const joi_1 = __importDefault(require("joi"));
const ability_1 = require("@casl/ability");
const type_1 = require("../type");
var AuthTokenType;
(function (AuthTokenType) {
    AuthTokenType["Bearer"] = "Bearer";
    AuthTokenType["Basic"] = "Basic";
})(AuthTokenType = exports.AuthTokenType || (exports.AuthTokenType = {}));
exports._authTokenType = joi_1.default.string()
    .trim()
    .valid(...Object.values(AuthTokenType));
exports.schemaBaseIdToken = {
    userId: type_1._id.required(),
    userLogin: type_1._userLogin.required(),
    localeCode: type_1._localeCode.required(),
    orgId: type_1._id.allow(null),
    appDomain: type_1._appDomain.allow(null),
    productId: type_1._id.allow(null),
    extended: type_1._object.allow(null)
};
exports.schemaRequestMetadata = joi_1.default.object()
    .keys({
// Customized keys here
})
    .pattern(joi_1.default.string(), joi_1.default.any());
function generateAbility(rules, vars, fieldMatcher, conditionsMatcher) {
    if (!fieldMatcher) {
        fieldMatcher = (fields) => (field) => fields.includes(field);
    }
    if (!conditionsMatcher) {
        conditionsMatcher = (matchConditions) => matchConditions;
    }
    return new ability_1.PureAbility(rules || [], { fieldMatcher, conditionsMatcher });
}
exports.generateAbility = generateAbility;
function renderCondition(conditions, vars) {
    if (conditions) {
        for (const key of Object.keys(conditions)) {
            // Support max 10 variables
            for (let i = 0; 10 > i; i++) {
                const renderedValue = renderValue(conditions[key], vars);
                if (undefined !== renderedValue) {
                    conditions[key] = renderedValue;
                    continue;
                }
                break;
            }
        }
    }
    return conditions;
}
exports.renderCondition = renderCondition;
function renderValues(object, vars) {
    if ('object' !== typeof object) {
        return (renderValue(object, vars) || object);
    }
    for (const key of Object.keys(object)) {
        object[key] = renderValues(object[key], vars);
    }
    return object;
}
exports.renderValues = renderValues;
function renderValue(value, vars) {
    if ('string' === typeof value) {
        const key = getKey(value);
        if (key) {
            const renderedValue = 'object' === typeof vars ? getValueByPath(vars, key) : vars(key);
            if ('string' === typeof renderedValue) {
                return value.replaceAll(`\${${key}}`, renderedValue);
            }
            else if ('number' === typeof renderedValue || 'boolean' === typeof renderedValue) {
                const keyValue = `\${${key}}`;
                // Return the whole value when exact match with key placeholder
                return keyValue === value ? renderedValue : value.replaceAll(keyValue, `${renderedValue}`);
            }
            else {
                return value.replaceAll(`\${${key}}`, '');
            }
        }
    }
}
function getKey(value) {
    const start = value.indexOf('${');
    if (0 <= start) {
        const end = value.indexOf('}', start + 2);
        if (start < end) {
            return value.slice(start + 2, end);
        }
    }
}
function getValueByPath(obj, path) {
    const paths = path.split('.');
    let value = obj;
    for (const key of paths) {
        value = value?.[key];
        if (undefined === value) {
            break;
        }
    }
    if ('object' !== typeof value) {
        return value;
    }
}
exports.getValueByPath = getValueByPath;
//# sourceMappingURL=auth-type.js.map