"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigBase = void 0;
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_utils_1 = require("cm-libs-utils");
class ConfigBase {
    static self;
    validators = {};
    static get Instance() {
        return this.self || (this.self = new ConfigBase());
    }
    static setValidator(key, validator) {
        const self = this.Instance;
        if (key) {
            if (validator) {
                self.validators[key] = validator;
            }
            else {
                delete self.validators[key];
            }
        }
        return this;
    }
    static clearValidator() {
        if (this.self) {
            this.self.validators = {};
        }
        return true;
    }
    static get(key, validator) {
        const self = this.Instance;
        if (!validator) {
            validator = self.validators[key];
        }
        else {
            if (!self.validators[key]) {
                self.validators[key] = validator;
            }
        }
        if (!validator) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_CONFIGURED, `Validator not configured for environment: ${key}`, { key });
        }
        let value = process.env[key];
        if ((0, cm_libs_utils_1.isUndefinedOrNull)(value)) {
            for (const [, argv] of process.argv.entries()) {
                const index = argv.indexOf('=');
                if (0 <= index && key === argv.substring(0, index)) {
                    value = argv.substring(index + 1);
                    break;
                }
            }
        }
        try {
            return validator(value);
        }
        catch (error) {
            if (undefined === value) {
                (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_FOUND, `Missing environment: ${key}`, { key });
            }
            const { message } = (0, cm_libs_context_aware_error_1.describeError)(error);
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_FORMAT, `${key}: ${message}`, { key, error });
        }
    }
}
exports.ConfigBase = ConfigBase;
//# sourceMappingURL=config-base.js.map