"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainEventCodec = void 0;
const cm_schema_event_1 = require("cm-schema-event");
const domain_schema_1 = require("./domain-schema");
const domain_event_cipher_1 = require("./domain-event-cipher");
class DomainEventCodec {
    schemaMessage;
    cipher;
    constructor(cipher, disableSchemaRegistry = false) {
        this.cipher = cipher;
        // Load schema from the central schema registry
        this.schemaMessage = new domain_schema_1.DomainSchema(cm_schema_event_1.Schema.DomainEventMessage, disableSchemaRegistry);
    }
    async encode(message) {
        const encodedMessage = await this.schemaMessage.encode(message);
        const cipher = this.cipher || domain_event_cipher_1.DomainEventCipher.instance;
        return cipher.encrypt(encodedMessage);
    }
    async decode(encodedBuffer) {
        const cipher = this.cipher || domain_event_cipher_1.DomainEventCipher.instance;
        const encodedMessage = await cipher.decrypt(encodedBuffer);
        return this.schemaMessage.decode(encodedMessage);
    }
}
exports.DomainEventCodec = DomainEventCodec;
//# sourceMappingURL=domain-event-codec.js.map