import { ContextAwareError } from 'cm-libs-context-aware-error';
import { EachMessagePayload, KafkaConfig } from 'cm-libs-kafka';
import { DomainEventMessage } from 'cm-schema-event';
import { DomainEventCipher } from './domain-event-cipher';
import { Registry } from './domain-schema';
export declare function initDomainEventConsumer<T = unknown, M = unknown>(topic: string, appendOrdinalIndex?: boolean, cipher?: DomainEventCipher, disableSchemaRegistry?: Registry | boolean | null): DomainEventConsumer<T, M>;
export declare function extractOrdinalIndex(hostname: string): string;
export type DomainEventHandler<T, M> = (message: DomainEventMessage<T, M>, payload: EachMessagePayload) => Promise<void>;
export type ExceptionHandler<T, M> = (error: Error | ContextAwareError, message: DomainEventMessage<T, M>, payload: EachMessagePayload) => Promise<void>;
export declare class DomainEventConsumer<T = unknown, M = unknown> {
    private config;
    readonly groupId: string;
    readonly topic: string;
    readonly heartbeatThreshold: number;
    private _consumer;
    private domainEvent;
    constructor(config: string | KafkaConfig, groupId: string, topic: string, heartbeatThreshold?: number, cipher?: DomainEventCipher, disableSchemaRegistry?: Registry | boolean | null);
    term(): Promise<void>;
    subscribe(handler: DomainEventHandler<T, M>, exceptionHandler?: ExceptionHandler<T, M>): Promise<void>;
    unsubscribe(): Promise<void>;
    getOffset(partition?: number): Promise<import("kafkajs").FetchOffsetsPartition[]>;
    get pulseTimestamp(): number;
    get consumer(): import("kafkajs").Consumer;
    get ready(): boolean;
}
