"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainEvent = void 0;
const domain_schema_1 = require("./domain-schema");
const domain_event_codec_1 = require("./domain-event-codec");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
class DomainEvent {
    codec;
    schema;
    metadataSchema;
    constructor(schema, metadataSchema, cipher, disableSchemaRegistry = false) {
        this.codec = new domain_event_codec_1.DomainEventCodec(cipher, disableSchemaRegistry);
        this.schema = new domain_schema_1.DomainSchema(schema, disableSchemaRegistry);
        this.metadataSchema = new domain_schema_1.DomainSchema(metadataSchema, disableSchemaRegistry);
    }
    async encode({ eventType, eventId, correlationId, correlationTopic, timestamp, body, metadata }) {
        if (body && !this.schema.isEnabled) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_CONFIGURED, 'Schema is not configured', { eventType });
        }
        if (metadata && !this.metadataSchema.isEnabled) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_CONFIGURED, 'Metadata schema is not configured', { eventType });
        }
        const eventMessage = {
            eventType,
            eventId,
            correlationId,
            correlationTopic,
            timestamp,
            body: body && this.schema.isEnabled ? await this.schema.encode(body) : undefined,
            metadata: metadata && this.metadataSchema.isEnabled ? await this.metadataSchema.encode(metadata) : undefined
        };
        return this.codec.encode(eventMessage);
    }
    async decode(encodedMessage) {
        const { eventType, eventId, correlationId, correlationTopic, timestamp, body, metadata } = await this.codec.decode(encodedMessage);
        const message = {
            eventType,
            eventId,
            correlationId,
            correlationTopic,
            timestamp,
            body: body ? await this.schema.decode(body) : undefined,
            metadata: metadata ? await this.metadataSchema.decode(metadata) : undefined
        };
        return message;
    }
    get schemaRegistryId() {
        return this.schema.registryId;
    }
    get metadataSchemaRegistryId() {
        return this.metadataSchema?.registryId;
    }
}
exports.DomainEvent = DomainEvent;
//# sourceMappingURL=domain-event.js.map