"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.authBasicTokenMiddleware = void 0;
const joi_1 = __importDefault(require("joi"));
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const type_1 = require("../type");
const auth_1 = require("../auth");
const http_auth_1 = require("./http-auth");
function authBasicTokenMiddleware(req, res, next) {
    const token = req.get('authorization');
    if (token && token.startsWith('Basic ')) {
        const auth = Buffer.from(token.replace('Basic ', ''), 'base64').toString('utf8');
        const loginInfo = auth.split(':');
        if (2 === loginInfo.length) {
            const ability = (0, auth_1.generateAbility)();
            const authToken = (0, http_auth_1.getAuthToken)(req) || { type: null, info: { token: null, ability } };
            authToken.type = auth_1.AuthTokenType.Basic;
            authToken.info.token = token;
            authToken.info.ability = ability;
            const basic = {
                username: loginInfo[0],
                password: loginInfo[1],
                orgId: undefined,
                appDomain: undefined,
                productId: undefined
            };
            authToken.authMetadata = { basic };
            const orgId = req.get('x-org-id');
            if (undefined !== orgId) {
                const appDomain = req.get('x-app-domain');
                const productId = req.get('x-product-id');
                if (undefined !== appDomain && undefined !== productId) {
                    const hasAbility = req.get('x-has-ability');
                    const value = validate(orgId, appDomain, productId, hasAbility);
                    const basic = {
                        ...authToken.authMetadata.basic,
                        orgId: value.orgId,
                        appDomain: value.appDomain,
                        productId: value.productId,
                        hasAbility: value.hasAbility
                    };
                    authToken.authMetadata = { basic };
                }
            }
            (0, http_auth_1.setAuthToken)(req, authToken);
        }
    }
    return next();
}
exports.authBasicTokenMiddleware = authBasicTokenMiddleware;
function validate(orgId, appDomain, productId, hasAbility) {
    const data = { orgId, appDomain, productId };
    if (hasAbility) {
        data['hasAbility'] = JSON.parse(hasAbility);
    }
    const { error, value } = joi_1.default.object()
        .keys({
        orgId: type_1._idString.required(),
        appDomain: type_1._appDomain.required(),
        productId: type_1._idString.required(),
        hasAbility: type_1._rule
    })
        .validate(data);
    if (error) {
        (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.INVALID_REQUEST, {
            function: 'authBasicTokenMiddleware',
            details: error.details,
            text: { input: error.details?.[0]?.context?.peers || error.details?.[0]?.context?.key }
        });
    }
    return value;
}
//# sourceMappingURL=http-auth-basic.js.map