"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getIp = exports.errorMiddleware = exports.finalMiddleware = exports.goodRes = void 0;
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_utils_1 = require("cm-libs-utils");
function goodRes(res, data, statusCode, raw = false) {
    const resp = {
        data: undefined === data ? null : data
    };
    if (!raw) {
        if (statusCode) {
            res.status(statusCode).json(resp);
        }
        else {
            res.json(resp);
        }
    }
    else {
        if (statusCode) {
            res.status(statusCode).send(data);
        }
        else {
            res.send(data);
        }
    }
}
exports.goodRes = goodRes;
/* eslint-disable @typescript-eslint/no-unused-vars */
function finalMiddleware(req, res, next) {
    next((0, cm_libs_context_aware_error_1.createError)(cm_libs_context_aware_error_1.errors.NOT_FOUND, 'Resource not found', { statusCode: 404 }));
}
exports.finalMiddleware = finalMiddleware;
function errorMiddleware(error, req, res, next) {
    const { code, message, reason } = (0, cm_libs_context_aware_error_1.describeError)(error);
    (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => {
        const elapsed = req['start'] ? (0, cm_libs_utils_1.timeElapsed)(req['start']).toString() : '-';
        return `RES: ${req.method} ${req.url} ${res.statusCode}, elapsed: ${elapsed + ' ms'}, ERROR: ${reason}`;
    });
    res.status(error.statusCode || 500).send({
        errors: [{ message, extensions: { code } }]
    });
}
exports.errorMiddleware = errorMiddleware;
function getIp(req) {
    // The X-Forwarded-For header might contains multiple values that separated by ","
    const xForwardedFor = req?.get('x-forwarded-for')?.split(',')?.[0] || req?.get('x-forwarded-for');
    return (xForwardedFor || req?.get('x-real-ip') || '0.0.0.0');
}
exports.getIp = getIp;
//# sourceMappingURL=http-handler.js.map