"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SecureLog = void 0;
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_utils_1 = require("cm-libs-utils");
const lodash_1 = __importDefault(require("lodash"));
const config_base_1 = require("../config-base");
class SecureLog {
    static LOG_SIZE = config_base_1.ConfigBase.get('LOG_SIZE', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkPositiveNumber, 1024));
    static #MASKING_LOG = config_base_1.ConfigBase.get('MASKING_LOG', (x) => {
        x = x ? JSON.parse(x) : [];
        if (!Array.isArray(x))
            throw new Error('invalid Array');
        return x;
    });
    static log(level, value, description = 'Object') {
        if (lodash_1.default.isObject(value)) {
            if (!lodash_1.default.isEmpty(value)) {
                const bytes = this.#getSizeInBytes(value);
                if (bytes >= this.LOG_SIZE) {
                    (0, cm_libs_context_aware_logger_1.log)(level, () => `${description} is approximately ${bytes} B, better don't log`);
                }
                else {
                    if (this.#MASKING_LOG.length > 0)
                        (0, cm_libs_context_aware_logger_1.log)(level, () => `${description}: ${JSON.stringify(this.#maskingPrint(value, this.#MASKING_LOG))}`);
                    else
                        (0, cm_libs_context_aware_logger_1.log)(level, () => `${description}: ${JSON.stringify(value)}`);
                }
            }
            else {
                (0, cm_libs_context_aware_logger_1.log)(level, () => `${description}: ${JSON.stringify(value)}`);
            }
        }
        else
            (0, cm_libs_context_aware_logger_1.log)(level, () => `${description}: ${value}`);
    }
    static #maskingPrint(value, arrayToMask) {
        const json = lodash_1.default.cloneDeep(value);
        if (!json || typeof json === 'string')
            return '';
        lodash_1.default.keysIn(json).forEach((key) => {
            this.#deepCheck(json, key, arrayToMask);
        });
        return json;
    }
    static #deepCheck(json, key, arrayToMask) {
        if ((typeof json[key] === 'object' || typeof json[key] === 'string') && arrayToMask.includes(key)) {
            json[key] = '[Masked Value!!!]';
        }
        else if (Array.isArray(json[key])) {
            json[key].forEach((arrayKey) => {
                this.#maskingPrint(arrayKey, arrayToMask);
            });
        }
    }
    static #getSizeInBytes(obj) {
        let str = null;
        if (typeof obj === 'string') {
            // If obj is a string, then use it
            str = obj;
        }
        else {
            // Else, make obj into a string
            str = JSON.stringify(obj);
        }
        // Get the length of the Uint8Array
        return new TextEncoder().encode(str).length;
    }
    static logSizeInBytes(description, obj) {
        const bytes = this.#getSizeInBytes(obj);
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => `${description} is approximately ${bytes} B`);
    }
}
exports.SecureLog = SecureLog;
//# sourceMappingURL=secure-log.js.map