"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseModel = exports.gqlSchemaBrowsePropertiesOutput = exports.gqlSchemaBrowseOptions = exports._browseOptions = exports.gqlSchemaBrowsePagination = exports._pagination = exports._pageSize = exports._page = exports.gqlSchemaIntFilter = exports.gqlSchemaStringFilter = exports.gqlSchemaDateFilter = exports.gqlSchemaQueryMode = exports.gqlSchemaSortOrder = void 0;
const joi_1 = __importDefault(require("joi"));
const graphql_tag_1 = require("graphql-tag");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const type_1 = require("../type");
exports.gqlSchemaSortOrder = (0, graphql_tag_1.gql) `
    enum SortOrder {
        asc
        desc
    }
`;
exports.gqlSchemaQueryMode = (0, graphql_tag_1.gql) `
    enum QueryMode {
        insensitive
        default
    }
`;
exports.gqlSchemaDateFilter = (0, graphql_tag_1.gql) `
    input DateFilterInput {
        equals: String
        lt: String
        lte: String
        gt: String
        gte: String
    }
`;
exports.gqlSchemaStringFilter = (0, graphql_tag_1.gql) `
    input StringFilterInput {
        in: [String]
        notIn: [String]
        equals: String
        contains: String
        startsWith: String
        endsWith: String
        mode: QueryMode
    }
`;
exports.gqlSchemaIntFilter = (0, graphql_tag_1.gql) `
    input IntFilterInput {
        equals: Int
        in: [Int]
        notIn: [Int]
        lt: Int
        lte: Int
        gt: Int
        gte: Int
    }
`;
exports._page = joi_1.default.number().min(1);
exports._pageSize = joi_1.default.number().min(0);
exports._pagination = joi_1.default.object().keys({
    page: exports._page,
    pageSize: exports._pageSize
});
exports.gqlSchemaBrowsePagination = (0, graphql_tag_1.gql) `
    input BrowsePaginationInput {
        page: Int
        pageSize: Int
    }
`;
exports._browseOptions = joi_1.default.object().keys({
    speedMode: type_1._boolean
});
exports.gqlSchemaBrowseOptions = (0, graphql_tag_1.gql) `
    input BrowseOptionsInput {
        speedMode: Boolean
    }
`;
exports.gqlSchemaBrowsePropertiesOutput = (0, graphql_tag_1.gql) `
    type BrowsePropertiesOutput {
        page: Int!
        pageSize: Int!
        count: Int
    }
`;
class BaseModel {
    schemas;
    model;
    middleware;
    constructor(middleware, name, schemas) {
        this.schemas = schemas;
        this.middleware = middleware;
        this.model = middleware.client[name] || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_FOUND, 'Model is not found', { name });
    }
    async run(accessInfoAndOptions, promise) {
        const { disableRestriction, ...accessInfo } = accessInfoAndOptions;
        const context = {
            schemas: this.schemas,
            accessInfo,
            disableRestriction
        };
        return await this.middleware.run(context, promise);
    }
    async browse(params) {
        params ??= {};
        const pagination = params.pagination || {};
        const options = params.options || {};
        delete params.pagination;
        delete params.options;
        pagination.page = !(0 < pagination.page) ? 1 : pagination.page;
        pagination.pageSize ??= 10;
        if (0 < pagination.pageSize) {
            params.take = pagination.pageSize;
            params.skip = (pagination.page - 1) * pagination.pageSize;
        }
        const promises = [this.model['findMany'](params)];
        if (!(options.speedMode ??= false)) {
            promises.push(this.model['count']({ where: params.where }));
        }
        const results = await Promise.all(promises);
        const ret = {
            items: results[0],
            properties: options.speedMode ? pagination : { ...pagination, count: results[1] }
        };
        return ret;
    }
    get client() {
        return this.middleware.client;
    }
}
exports.BaseModel = BaseModel;
//# sourceMappingURL=model.js.map