import { PrismaClient } from '@prisma/client';
import Joi from 'joi';
import { AccessInfo } from '../auth';
import { BaseModel, BaseModelBrowseArgs } from '../model';
type Service = {
    entityName: string;
    model: any;
    schemaBrowse: Joi.ObjectSchema;
    schemaRead: Joi.ObjectSchema;
    schemaEdit: Joi.ObjectSchema;
    schemaAdd: Joi.ObjectSchema;
    schemaDelete: Joi.ObjectSchema;
    schemaActivate: Joi.ObjectSchema;
    schemaDeactivate: Joi.ObjectSchema;
    auditTrail?: boolean;
};
type RecordConfig = {
    includeInactive?: boolean;
};
export type NoInput = Record<string, never>;
export declare class BaseService<T, Browse, Read, Edit, Add, Delete, Activate, Deactivate> {
    readonly BASE_SERVICE_LOG: boolean;
    readonly BASE_SERVICE_FILTER_ORG: boolean;
    private readonly entityName;
    readonly model: BaseModel<T, PrismaClient, PrismaClient, BaseModelBrowseArgs> & {
        instance: BaseModel<T, PrismaClient, PrismaClient, BaseModelBrowseArgs>;
    };
    readonly schemaBrowse: Joi.ObjectSchema;
    readonly schemaRead: Joi.ObjectSchema;
    readonly schemaEdit: Joi.ObjectSchema;
    readonly schemaAdd: Joi.ObjectSchema;
    readonly schemaDelete: Joi.ObjectSchema;
    readonly schemaActivate: Joi.ObjectSchema;
    readonly schemaDeactivate: Joi.ObjectSchema;
    readonly auditTrail: boolean;
    readonly schemaNoInput: Joi.ObjectSchema;
    constructor({ entityName, model, schemaBrowse, schemaRead, schemaEdit, schemaAdd, schemaDelete, schemaActivate, schemaDeactivate, auditTrail }: Service);
    protected filterOrganization(_params: Browse | Read | Edit | Delete | Activate | Deactivate, _accessInfo: AccessInfo): void;
    protected filterRecordStatus(_params: Browse | Read | Edit | Delete | Activate | Deactivate, _accessInfo: AccessInfo): void;
    browse(params: Browse, accessInfo: AccessInfo, config?: RecordConfig): Promise<import("../model").BaseModelBrowseReturn<T>>;
    read(params: Read, accessInfo: AccessInfo, config?: RecordConfig): Promise<unknown>;
    edit(params: Edit, accessInfo: AccessInfo, config?: RecordConfig): Promise<any>;
    add(params: Add, accessInfo: AccessInfo): Promise<unknown>;
    delete(params: Delete, accessInfo: AccessInfo, config?: RecordConfig): Promise<unknown>;
    activate(params: Activate, accessInfo: AccessInfo, config?: RecordConfig): Promise<any>;
    deactivate(params: Deactivate, accessInfo: AccessInfo, config?: RecordConfig): Promise<any>;
}
export {};
