"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseService = void 0;
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_utils_1 = require("cm-libs-utils");
const joi_1 = __importDefault(require("joi"));
const config_base_1 = require("../config-base");
const secure_log_1 = require("../misc/secure-log");
const model_1 = require("../model");
class BaseService {
    BASE_SERVICE_LOG = config_base_1.ConfigBase.get('BASE_SERVICE_LOG', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkBoolean, false));
    BASE_SERVICE_FILTER_ORG = config_base_1.ConfigBase.get('BASE_SERVICE_FILTER_ORG', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkBoolean, true));
    entityName;
    model;
    schemaBrowse;
    schemaRead;
    schemaEdit;
    schemaAdd;
    schemaDelete;
    schemaActivate;
    schemaDeactivate;
    auditTrail;
    schemaNoInput;
    constructor({ entityName, model, schemaBrowse, schemaRead, schemaEdit, schemaAdd, schemaDelete, schemaActivate, schemaDeactivate, auditTrail = false }) {
        this.entityName = entityName;
        this.model = model;
        this.schemaBrowse = schemaBrowse;
        this.schemaRead = schemaRead;
        this.schemaEdit = schemaEdit;
        this.schemaAdd = schemaAdd;
        this.schemaDelete = schemaDelete;
        this.schemaActivate = schemaActivate;
        this.schemaDeactivate = schemaDeactivate;
        this.auditTrail = auditTrail;
        this.schemaNoInput = joi_1.default.object().unknown(false);
    }
    /// For Overriding Purpose ///
    // eslint-disable-next-line @typescript-eslint/no-empty-function, @typescript-eslint/no-unused-vars
    filterOrganization(_params, _accessInfo) { }
    // eslint-disable-next-line @typescript-eslint/no-empty-function, @typescript-eslint/no-unused-vars
    filterRecordStatus(_params, _accessInfo) { }
    async browse(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        const dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.browse(params));
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    //#endregion
    //#region Read
    async read(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        const dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.findUnique(params));
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    //#endregion
    //#region Edit
    async edit(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        let dbRetData;
        if (this.auditTrail) {
            const tempArgs = params ? { ...params } : {};
            const actionName = 'edit';
            dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update({
                ...params,
                data: {
                    timelines: {
                        create: {
                            action: actionName,
                            args: { ...tempArgs }
                        }
                    },
                    status: model_1.RecordStatus.INACTIVE
                }
            }));
        }
        else {
            dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update(params));
        }
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    //#endregion
    //#region Add
    async add(params, accessInfo) {
        if (this.auditTrail) {
            const tempArgs = params ? { ...params } : {};
            const actionName = 'add';
            params['data'].timelines = {
                create: {
                    action: actionName,
                    args: { ...tempArgs }
                }
            };
        }
        const dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.create(params));
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    //#endregion
    // #region Delete
    async delete(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        const dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.delete(params));
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    async activate(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        let dbRetData;
        if (this.auditTrail) {
            const tempArgs = params ? { ...params } : {};
            const actionName = 'activate';
            dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update({
                ...params,
                data: {
                    timelines: {
                        create: {
                            action: actionName,
                            args: { ...tempArgs }
                        }
                    },
                    status: model_1.RecordStatus.ACTIVE
                }
            }));
        }
        else {
            dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update({ ...params, data: { status: model_1.RecordStatus.ACTIVE } }));
        }
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
    async deactivate(params, accessInfo, config) {
        if (this.BASE_SERVICE_FILTER_ORG)
            this.filterOrganization(params, accessInfo);
        if (!config?.includeInactive)
            this.filterRecordStatus(params, accessInfo);
        let dbRetData;
        if (this.auditTrail) {
            const tempArgs = params ? { ...params } : {};
            const actionName = 'deactivate';
            dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update({
                ...params,
                data: {
                    timelines: {
                        create: {
                            action: actionName,
                            args: { ...tempArgs }
                        }
                    },
                    status: model_1.RecordStatus.INACTIVE
                }
            }));
        }
        else {
            dbRetData = await this.model.instance.run(accessInfo, () => this.model.instance.model.update({ ...params, data: { status: model_1.RecordStatus.INACTIVE } }));
        }
        if (this.BASE_SERVICE_LOG)
            secure_log_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, dbRetData, 'dbRetData');
        return dbRetData;
    }
}
exports.BaseService = BaseService;
//# sourceMappingURL=base.service.js.map