"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.baseHandler = void 0;
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const error_1 = require("../error");
const lodash_1 = __importDefault(require("lodash"));
async function baseHandler(thisObject, fn, schema, request, info) {
    try {
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DETAIL, () => `fn(${fn.name}) => Start`);
        const { error, value } = schema.validate(request, { stripUnknown: true });
        if (error) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_PARAMETER, error.message, {
                function: fn.name,
                details: error.details,
                text: { input: error.details?.[0]?.context?.peers || error.details?.[0]?.context?.key }
            });
        }
        return await fn.apply(thisObject, [value, info]);
    }
    catch (error) {
        let tmpError = JSON.stringify(error);
        if (lodash_1.default.isEmpty(tmpError) || tmpError === '{}' || tmpError === '') {
            tmpError = error;
        }
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.ERROR, () => `fn(${fn.name}) => Raw Error:${tmpError}`);
        if (!error_1.ErrorHandler.isEnabled) {
            (0, cm_libs_context_aware_error_1.throwFromError)(error);
        }
        const { code, message, statusCode } = await error_1.ErrorHandler.translate(error, error.statusCode, info?.headers?.localeCode);
        const params = {
            ...error.params,
            statusCode
        };
        (0, cm_libs_context_aware_error_1.throwError)(code, message || fallbackError(error?.reason), params, false, error.stack);
    }
}
exports.baseHandler = baseHandler;
function fallbackError(reason = '') {
    const found = reason.split(`\n`);
    return found.length >= 1 ? found[0] : undefined;
}
//# sourceMappingURL=validator.js.map