"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.authDirective = exports.authDirectiveTypeDefs = void 0;
const joi_1 = __importDefault(require("joi"));
const graphql_tag_1 = require("graphql-tag");
const utils_1 = require("@graphql-tools/utils");
const graphql_1 = require("graphql");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const common_1 = require("../common");
exports.authDirectiveTypeDefs = (0, graphql_tag_1.gql) `
    directive @auth(type: String!, flag: Boolean) on FIELD | FIELD_DEFINITION
`;
function authDirective(schema, directiveName = 'auth') {
    return (0, utils_1.mapSchema)(schema, {
        [utils_1.MapperKind.OBJECT_FIELD]: (fieldConfig) => {
            const authDirective = (0, utils_1.getDirective)(schema, fieldConfig, directiveName)?.[0];
            if (authDirective) {
                const { resolve = graphql_1.defaultFieldResolver } = fieldConfig;
                const { error, value } = joi_1.default.object()
                    .keys({
                    type: common_1._authTokenType.required(),
                    flag: common_1._boolean.default(false)
                })
                    .validate(authDirective);
                if (error) {
                    if (error instanceof graphql_1.GraphQLError) {
                        throw error;
                    }
                    const { message } = (0, cm_libs_context_aware_error_1.describeError)(error);
                    throw new graphql_1.GraphQLError(message, { originalError: error, extensions: { code: cm_libs_context_aware_error_1.errors.INVALID_REQUEST, originalError: error } });
                }
                fieldConfig.resolve = async (source, args, context, info) => {
                    try {
                        await (0, common_1.auth)(context?.req, value);
                        return await resolve(source, args, context, info);
                    }
                    catch (error) {
                        if (error instanceof graphql_1.GraphQLError) {
                            throw error;
                        }
                        const { code, message } = (0, cm_libs_context_aware_error_1.describeError)(error);
                        throw new graphql_1.GraphQLError(message, { originalError: error, extensions: { code, originalError: error } });
                    }
                };
            }
            return fieldConfig;
        }
    });
}
exports.authDirective = authDirective;
//# sourceMappingURL=auth.directive.js.map