/// <reference types="node" />
/// <reference types="node" />
/// <reference types="node" />
import { Socket } from 'net';
import { TLSSocket } from 'node:tls';
export { Socket };
export interface TcpConnection<T> {
    clientId: string;
    socket: Socket;
    tlsSocket?: TLSSocket;
    buffer: Buffer;
    isIdle: boolean;
    ref?: T;
}
export declare abstract class TcpBase<T> {
    protected connections: Record<string, TcpConnection<T>>;
    private _connectionCount;
    protected abstract onConnected(connection: TcpConnection<T>): Promise<void>;
    protected abstract onClose(connection: TcpConnection<T>, error: Error | undefined): Promise<void>;
    protected abstract onData(connection: TcpConnection<T>, data: Buffer): Promise<number>;
    term(): Promise<void>;
    protected addConnection(socket: Socket, tlsSocket: TLSSocket, clientId: string, ref?: T, keepAliveInterval?: number, idleTimeout?: number): Promise<TcpConnection<T>>;
    protected removeConnection(clientId: string): boolean;
    private onDataEvent;
    private onCloseEvent;
    protected get connectionCount(): number;
    protected static socketWrite(socket: Socket | TLSSocket, data: Buffer): Promise<void>;
    protected static resetIdleTimeout(socket: Socket | TLSSocket, idleTimeout: number): void;
}
