"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TcpClient = exports.TCP_CLIENT_PROTOCOL = void 0;
const node_tls_1 = __importDefault(require("node:tls"));
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const tcp_base_1 = require("./tcp-base");
__exportStar(require("./tcp-base"), exports);
var TCP_CLIENT_PROTOCOL;
(function (TCP_CLIENT_PROTOCOL) {
    TCP_CLIENT_PROTOCOL["TCP"] = "TCP";
    TCP_CLIENT_PROTOCOL["TLS"] = "TLS";
})(TCP_CLIENT_PROTOCOL = exports.TCP_CLIENT_PROTOCOL || (exports.TCP_CLIENT_PROTOCOL = {}));
class TcpClient extends tcp_base_1.TcpBase {
    hostname;
    port;
    connectTimeout;
    keepAliveInterval;
    idleTimeout;
    detectEarlyClosure;
    ref;
    protocol = TCP_CLIENT_PROTOCOL.TCP;
    tlsServerName;
    tlsCert;
    tlsKey;
    tlsCa;
    tlsOnCheckServerIdentity;
    tlsSession;
    constructor(hostname, port, connectTimeout, keepAliveInterval, idleTimeout, detectEarlyClosure) {
        super();
        this.hostname = hostname;
        this.port = port;
        this.connectTimeout = 0 < connectTimeout ? connectTimeout : 0;
        this.keepAliveInterval = 0 < keepAliveInterval ? keepAliveInterval : 0;
        this.idleTimeout = 0 < idleTimeout ? idleTimeout : 0;
        this.detectEarlyClosure = 0 < detectEarlyClosure ? detectEarlyClosure : 0;
    }
    initTls(cert, key, ca, serverName, onCheckServerIdentity) {
        this.protocol = TCP_CLIENT_PROTOCOL.TLS;
        this.tlsCa = ca;
        this.tlsServerName = serverName;
        this.tlsOnCheckServerIdentity = onCheckServerIdentity;
        if (cert) {
            this.tlsKey = key || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'Tls client key is required');
            this.tlsCert = cert;
        }
    }
    async connect(clientId, ref) {
        const socket = new tcp_base_1.Socket();
        try {
            await this.onConnect(clientId, this.hostname, this.port, this.connectTimeout);
            await new Promise((resolve, reject) => {
                try {
                    // Setup the timer for connection timeout (the duration of connecting)
                    socket.setTimeout(this.connectTimeout);
                    socket.once('timeout', () => socket.destroy(new Error('ETIMEDOUT')));
                    socket.once('error', (error) => {
                        return reject((0, cm_libs_context_aware_error_1.createError)(cm_libs_context_aware_error_1.errors.CONNECTION_ERROR, error.message, { ...error }));
                    });
                    let closed = false;
                    socket.once('close', () => {
                        closed = true;
                    });
                    socket.once('connect', async () => {
                        // Cater the misbehavior of istio for newly established connection get disconnected when remote side is not available
                        if (0 < this.detectEarlyClosure) {
                            await new Promise((resolve) => {
                                if (closed) {
                                    return resolve();
                                }
                                // We introduce some delay to monitor the connection closure before mark connection as connected
                                const timer = setTimeout(() => {
                                    return resolve();
                                }, this.detectEarlyClosure);
                                socket.once('close', () => {
                                    closed = true;
                                    clearTimeout(timer);
                                    return resolve();
                                });
                            });
                        }
                        socket.removeAllListeners();
                        if (closed) {
                            return reject((0, cm_libs_context_aware_error_1.createError)(cm_libs_context_aware_error_1.errors.CONNECTION_ERROR, 'Connection reset'));
                        }
                        return resolve(socket);
                    });
                    socket.connect(this.port, this.hostname);
                }
                catch (error) {
                    return reject((0, cm_libs_context_aware_error_1.createError)(cm_libs_context_aware_error_1.errors.CONNECTION_ERROR, error.message, { ...error }));
                }
            });
            const tlsSocket = TCP_CLIENT_PROTOCOL.TLS === this.protocol ? await this.secureConnect(socket) : undefined;
            return super.addConnection(socket, tlsSocket, clientId, ref, this.keepAliveInterval, this.idleTimeout);
        }
        catch (error) {
            await this.raiseConnectionError(socket, clientId, error);
            throw error;
        }
    }
    async secureConnect(socket) {
        const options = {
            socket
        };
        if (this.tlsCa) {
            options.ca = this.tlsCa;
        }
        if (this.tlsCert) {
            options.cert = this.tlsCert;
            options.key = this.tlsKey;
        }
        if (this.tlsServerName) {
            options.servername = this.tlsServerName;
        }
        if (this.tlsOnCheckServerIdentity) {
            options.checkServerIdentity = this.tlsOnCheckServerIdentity;
        }
        if (this.tlsSession) {
            options.session = this.tlsSession;
        }
        return new Promise((resolve, reject) => {
            try {
                const tlsSocket = node_tls_1.default.connect(options, () => {
                    if (tlsSocket.authorized) {
                        this.tlsSession = tlsSocket.getSession();
                        return resolve(tlsSocket);
                    }
                    return reject(tlsSocket.authorizationError);
                });
                tlsSocket.on('error', (error) => {
                    return reject(error);
                });
            }
            catch (error) {
                socket.destroy(error);
                return reject(error);
            }
        });
    }
    async raiseConnectionError(socket, clientId, error) {
        const connection = {
            clientId,
            socket,
            buffer: Buffer.alloc(0),
            isIdle: true,
            ref: undefined
        };
        try {
            await this.onClose(connection, error);
        }
        catch (error) {
            // Ignore this error
        }
    }
}
exports.TcpClient = TcpClient;
//# sourceMappingURL=tcp-client.js.map