"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TcpServer = void 0;
const net_1 = __importDefault(require("net"));
const nanoid_1 = require("nanoid");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const tcp_base_1 = require("./tcp-base");
class TcpServer extends tcp_base_1.TcpBase {
    server;
    _keepAliveInterval;
    _idleTimeout;
    async init(params) {
        const { keepAliveInterval, idleTimeout } = params;
        this._keepAliveInterval = 0 < keepAliveInterval ? keepAliveInterval : 5000;
        this._idleTimeout = 0 < idleTimeout ? idleTimeout : 0;
        this.server = net_1.default.createServer((socket) => {
            const clientId = (0, nanoid_1.nanoid)();
            super.addConnection(socket, null, clientId, undefined, this._keepAliveInterval, this._idleTimeout);
        });
        return this;
    }
    async term() {
        if (this.server) {
            this.server.close(); // Stop accept new connection
        }
        await super.term();
    }
    async run(port, hostname) {
        if (!this.server) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'TcpServer initialization is required');
        }
        if (!(0 < port)) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_PARAMETER, 'Host port is required');
        }
        return new Promise((resolve) => {
            this.server?.listen(port, hostname || '0.0.0.0', () => {
                return resolve(`${hostname ? hostname : '*'}:${port}`);
            });
        });
    }
    get keepAliveInterval() {
        return this._keepAliveInterval;
    }
    get idleTimeout() {
        return this._idleTimeout;
    }
}
exports.TcpServer = TcpServer;
//# sourceMappingURL=tcp-server.js.map